﻿Option Strict Off
Option Explicit On

Imports System
Imports System.Windows.Forms
Imports System.Text.Encoding
Imports System.IO.Ports.SerialPort


Public Class Form1

    'Comports
    Public ports As String() = GetPortNames()
    Public port As String = ""
    Declare Sub Beep Lib "kernel32.dll" (ByVal tone As Integer, ByVal dauer As Integer)


    'Sleep Funktion in ms
    Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)


    'Empfangs Buffer für die RS232
    Dim Buffer As String

    'Sende Variablen
    Dim Addr As Byte
    Dim Command1 As Byte
    Dim Command2 As Byte
    Dim CRC As Byte
    Dim Speed As Byte

    'PRO-BOT128 Adressen
    Const Robo_Addr = 42

    Private Sub Button10_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button10.Click
        TextBox1.Clear()
    End Sub

    Private Sub ComboBox1_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBox1.SelectedIndexChanged

        'Comport Auswahl, ist kein Comport eingestellt "Connect" und "Disconnect" sperren
        If ComboBox1.SelectedItem <> "" Then
            Button1.Enabled = True
        Else
            Button1.Enabled = False
            Button2.Enabled = False
        End If

    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        'Roboter Information
        TextBox1.Text = _
                        "Roboter    : PRO-BOT128" & vbNewLine & _
                        "Controller : C-Control PRO Mega128" & vbNewLine & _
                        "================================="

        'Verfügbare Comports abfragen und in die Combobox schreiben
        For Each Me.port In ports
            ComboBox1.Items.Add(port)
        Next port

        'Comport bereits eingestellt?
        'Buttons freigeben oder sprerren
        If ComboBox1.Text <> "" Then
            Button1.Enabled = True
            Button2.Enabled = False
            Speed = TrackBar1.Value
        Else
            Button1.Enabled = False
            Button2.Enabled = False
        End If

        Speed = TrackBar1.Value

    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing

        'Abfrage, ob das Programm wirklich beendet werden soll
        If MsgBox("Wollen Sie das Programm wirklich beenden?", vbYesNo) = vbNo Then
            ' Falls nein: Schließen der Form durch 
            ' Setzen von e.Cancel auf True verhindern
            e.Cancel = True
            Exit Sub
        End If

    End Sub

    Private Sub Send_Command(ByVal Addr As Integer, ByVal Command1 As Integer, ByVal Command2 As Integer, ByVal Command3 As Integer)

        'Prüfsumme berechnen mit XOR
        CRC = Addr Xor Command1 Xor Command2 Xor Command3

        'Sende Kommandos in die Textboxen ausgeben
        TextBox1.AppendText(Addr & "<>" & Command1 & "<>" & Command2 & "<>" & Command3 & "<>" & CRC & vbNewLine)

        'Comport offen? Ja dann senden!
        If SerialPort1.IsOpen = True Then
            Control.CheckForIllegalCrossThreadCalls = False
            SerialPort1.DiscardOutBuffer()
            SerialPort1.WriteLine(Chr(Addr) & Chr(Command1) & Chr(Command2) & Chr(Command3) & Chr(CRC))
        End If

    End Sub

    Private Sub SerialPort1_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived

        'RS232 Daten Empfang
        'Empfangener Text wird im Debug Monitor angezeigt
        Control.CheckForIllegalCrossThreadCalls = False
        Buffer = (SerialPort1.ReadExisting)
        TextBox1.AppendText(Buffer)

    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click

        'Comport verbinden
        Try

            'Auswahl der Combobox übernehmen und Schnittstelle öffnen, darauf gleich Status vom BOT abfragen
            SerialPort1.PortName = ComboBox1.Text
            SerialPort1.BaudRate = 115200 '19200
            SerialPort1.Encoding = System.Text.Encoding.Default
            SerialPort1.Open()

            'Status abrufen
            TextBox1.Text = ""
            Send_Command(Robo_Addr, 4, 1, 1)

            'Buttons setzen
            Button1.Enabled = False
            Button2.Enabled = True
            ComboBox1.Enabled = False
            GroupBox1.Enabled = True

        Catch ex As Exception

            'Info wenn ein Schnittstellenfehler aufgetretten ist und Buttons setzen
            MessageBox.Show("Schnittstellen-Fehler!" + e.ToString _
                          , "Exception error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Button1.Enabled = True
            Button2.Enabled = False
            ComboBox1.Enabled = True
            GroupBox1.Enabled = False

        End Try

    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click

        'Schnittstelle schließen und Buttons setzen
        Button1.Enabled = True
        Button2.Enabled = False
        ComboBox1.Enabled = True
        SerialPort1.Close()

    End Sub

    Private Sub Button8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button8.Click

        ' Abfrage, ob der Roboter wirklich resetet werden soll
        If MsgBox("Wollen Sie den Roboter wirklich zurücksetzen?", vbYesNo) = vbNo Then
            ' Falls nein: Schließen der Form durch 
            ' Setzen von e.Cancel auf True verhindern
            Exit Sub
        End If

        'Ok, jetzt wird der Roboter resetet
        TextBox1.Text = ""
        Send_Command(Robo_Addr, 255, 128, 0) 'reset
        Speed = TrackBar1.Value
        Sleep(1000)

    End Sub

    Private Sub TrackBar1_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBar1.Scroll
        Speed = TrackBar1.Value
    End Sub

    Private Sub Button3_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button3.MouseDown
        Send_Command(Robo_Addr, 1, 1, Speed)
    End Sub

    Private Sub Button3_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button3.MouseUp
        Send_Command(Robo_Addr, 1, 255, 0)
    End Sub

    Private Sub Button5_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button5.MouseDown
        Send_Command(Robo_Addr, 1, 2, Speed)
    End Sub

    Private Sub Button5_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button5.MouseUp
        Send_Command(Robo_Addr, 1, 255, 0)
    End Sub

    Private Sub Button6_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button6.MouseDown
        Send_Command(Robo_Addr, 1, 3, Speed)
    End Sub

    Private Sub Button6_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button6.MouseUp
        Send_Command(Robo_Addr, 1, 255, 0)
    End Sub

    Private Sub Button7_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button7.MouseDown
        Send_Command(Robo_Addr, 1, 4, Speed)
    End Sub

    Private Sub Button7_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Button7.MouseUp
        Send_Command(Robo_Addr, 1, 255, 0)
    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        Send_Command(Robo_Addr, 1, 255, 0)
    End Sub

    Private Sub Button9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button9.Click
        Send_Command(Robo_Addr, 4, 1, 0)
        Sleep(500) 'kurze pause 
    End Sub

    Private Sub TextBox1_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBox1.TextChanged
        'Immer den Focus auf den Debug Monitor wenn Zeichen empfangen werden
        'Dadurch wird der Curser angezeigt ;-)
        TextBox1.Focus()
    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click

    End Sub
End Class
