/*******************************************************************************
 Autor: Ulli Sommer
 Subsumtion
*******************************************************************************/


//Multithreading Threads
//Threads zur Sensorabfrage, Verhaltenskontroller usw.
#define THREAD_CHECK_ACS            1
#define THREAD_BEHAVIOUR_CONTROLLER 2

//Threads fr Verhaltensweisen
#define THREAD_BEHAVIOUR_CRUISE     3
#define THREAD_BEHAVIOUR_ESCAPE     4

//Zustand des Verhaltens (bergabeparameter)
#define STATE                       0
#define COMMAND_SPEED               1
#define COMMAND_DIR                 2

//Bewegungs Kommandos
#define LEFT                        1
#define RIGHT                       2
#define BWD                         3
#define FWD                         4
#define STOP                        5

//Kommando Array (STATE, COMMAND_SPEED usw...)
//fgen wir einen neuen Verhaltenszustand hinzu, mssen wir das Arry vergrern
#define COMMAND_SIZE                3

//Task Zustand
#define IDLE                        0
#define ACTIVE                      1

//---[ Sensoren abfragen ]------------------------------------------------------

//ACS (Anti Collisions System)
#define ACS_DETECT    0
#define ACS_NO_DETECT 1
byte ACS_LEFT_SENSOR, ACS_RIGHT_SENSOR;

void thread_check_sensors(void)
{
    do
    {
       ACS_LEFT_SENSOR=ACS_LEFT();
       ACS_RIGHT_SENSOR=ACS_RIGHT();
       Thread_Delay(10);
    }while(1);
}


//---[ CRUISE VERHALTEN ]-------------------------------------------------------

//Hier fhrt unser Roboter einfach immer geradeaus...
//Dieses Verhalten ist immer aktiv, besitzt aber die niedrigste Prioritt

//Fahrgeschwindigkeit (128 bis 200)
#define CRUISE_SPEED 180

//Array anlegen fr cruise... gre ist COMMAND_SIZE
byte cruise[COMMAND_SIZE];

void behaviour_cruise(void)
{
    cruise[STATE]=ACTIVE;
    cruise[COMMAND_SPEED]=CRUISE_SPEED;
    cruise[COMMAND_DIR]=FWD;

    do
    {

        Thread_Delay(10); //den anderen Tasks etwas Zeit geben...

    }while(1);
}

//---[ ESCAPE VERHALTEN ]-------------------------------------------------------

//Geschwindigkeits Vorgaben
#define ESCAPE_BWD_SPEED     160
#define ESCAPE_ROTATE_SPEED  160

//Zeitvorgaben fr die Bewegung (var * 10ms)
#define ROTATE_TIME          50
#define DRIVE_TIME           100

//Zustand
#define ESCAPE_FRONT         1
#define ESCAPE_LEFT          2
#define ESCAPE_RIGHT         3

byte escape[COMMAND_SIZE];

void behaviour_escape(void)
{
    static word time_counter, move_time;

    //Verhalten initialisieren
    escape[STATE]=IDLE;
    escape[COMMAND_SPEED]=128;
    escape[COMMAND_DIR]=FWD;

    do
    {
        //---> Frontalzusammensto

        //Beide Sensoren haben angesprochen
        //Das Hindernis muss voruns liegen!
        if((ACS_LEFT_SENSOR==ACS_DETECT)&&(ACS_RIGHT_SENSOR==ACS_DETECT))

        {
           //Zustand auf Frontalzusammensto ndern
           escape[STATE]=ESCAPE_FRONT;
           move_time=DRIVE_TIME;
        }

        //---> Hindernis links

        //Der linke Sensor hat ausgelst
        //Das Hindernis muss links von uns liegen

        else if(ACS_LEFT_SENSOR==ACS_DETECT)             // Links
        {
             //Falls nicht schon erkannt wurde das beide
             //Tastsensoren aktiviert waren, kann der
             //Zustand auf Zusammensto links gendert werden

             if(escape[STATE] != ESCAPE_FRONT)
             {
                escape[STATE]=ESCAPE_LEFT;
                move_time=ROTATE_TIME;
             }

             //Wenn dieser Zustand aktiviert wird,
             //ist der Roboter mit dem linken Bumper
             //gegen ein Hindernis gestoen. Dass muss aber
             //noch nichts heissen, es kann sein, dass kurze
             //Zeit spter doch noch ESCAPE_FRONT aktiviert
             //wird. Der rechte Bumper knnte ein ganz klein
             //wenig spter aktiviert werden...
             //Mit der Abfrage oben if(escape(STATE) <> ESCAPE_FRONT)
             //wird sichergestellt, dass auch der Zustand FRONT gehalten
             //wird wenn einer der Bumper wieder inaktiv wird.

        }

        else if(ACS_RIGHT_SENSOR==ACS_DETECT)            // Rechts

        {

            //---> Hindernis rechts

            //Der rechte Sensor hat ausgelst
            //Das Hindernis muss rechts von uns liegen

             if(escape[STATE] != ESCAPE_FRONT)
             {
                escape[STATE]=ESCAPE_RIGHT;
                move_time=ROTATE_TIME;
             }
        }

        //---> Escape wurde aktiviert?

        if(escape[STATE] != IDLE) // Verhalten ist aktiv? Dann Ausweichen...
        {
              //Verhalten ist aktiv? Dann Ausweichen...
              //Der Thread wird nach jedem Durchlauf fr etwa 10ms pausiert.
              //(auch lnger wenn andere Threads laufen! Das ist also SEHR
              //ungenau und nur als grobe Timing Funktion verwendbar!)
              //Wir zhlen bei jedem Durchlauf einfach eine Timer-Variable
              //hoch um fr bestimmte Zeitintervalle verschiedene Bewegungen
              //durchzufhren...

              //move_time * 10ms lang Rckwrts fahren!
              if(time_counter < move_time)

              {
                 escape[COMMAND_SPEED]=ESCAPE_BWD_SPEED;     //Geschwindigkeit setzen
                 escape[COMMAND_DIR]=BWD;                    //Rckwrts fahren

              }

              //weitere move_time * 10ms lang rotieren!
              else if(time_counter < (2*move_time))
              {
                  escape[COMMAND_SPEED]=ESCAPE_ROTATE_SPEED; //Geschwindigkeit setzen

                   //Je nachdem wo die Kollision stattgefunden hat, wird
                   //die Ausweichrichtung gesetzt:
                  if(escape[STATE]==ESCAPE_FRONT)            //Frontal...
                  {
                     escape[COMMAND_DIR] = LEFT;
                  }

                  //Hier knnte man auch intelligenter ausweichen...
                  //z.B. abhngig davon welcher Bumper zuletzt aktiv
                  //war o....

                  else if(escape[STATE]==ESCAPE_LEFT)        //Links...
                  {
                          escape[COMMAND_DIR]=RIGHT;
                  }

                  else if(escape[STATE]==ESCAPE_RIGHT)       //Rechts...
                  {
                          escape[COMMAND_DIR]=LEFT;
                  }

              }
              else  // Nach insgesamt (2 * move_time * 10ms) ...
              {
                  //wieder zurck in den Ruhezustand!
                  escape[STATE]=IDLE;
              }

              //Zhlvariable fr die Zeit...
              time_counter++;
        }
        else

              time_counter = 0;

        Thread_Delay(1);  // 10ms warten

    }while(1);
}


//---[ Fahrkommando bergeben ]-------------------------------------------------
void moveCommand(byte command[])
{
    byte speed, move;
    speed=command[COMMAND_SPEED];
    move=command[COMMAND_DIR];

    switch(move)
    {

           //vorwrts
           case FWD:
                DRIVE_FWD(speed,0);
                FLL_OFF();
                FLR_OFF();
                BLL_OFF();
                BLR_OFF();
                break;

           //rckwrts
           case BWD:
                DRIVE_BWD(speed,0);
                BLL_ON();
                BLR_ON();

           //links drehen
           case LEFT:
                TURN_LEFT(speed,0);
                FLL_ON();
                FLR_OFF();
                break;

           //rechts drehen
           case RIGHT:
                TURN_RIGHT(speed,0);
                FLL_OFF();
                FLR_ON();
                break;

           //stoppen
           case STOP:
                MOTOR_STOP();
                FLL_OFF();
                FLR_OFF();
                break;

    }

}


//---[ Verhaltens Kontroller mit Prioritt ]------------------------------------
void behaviour_controller(void)
{
    do
    {
        //Threadumschaltung sperren!
        //Das ist wichtig damit nicht whrend die Kommandos ausgefhrt
        //werden noch etwas an den Kommando Arrays gendert wird.

        Thread_Lock(1);

        //Hier werden die einzelnen Verhalten der Reihe nach abgefragt.
        //Nochmal die Erluterung: Es kann nur EINES der Verhalten die
        //Kontrolle ber den Roboter bernehmen!
        //Wenn mehrere Verhalten aktiv sind, entscheidet die Prioritt
        //der Verhalten welches die Kontrolle bekommt.
        //Da hier ein if-else-if-else Konstrukt verwendet wird, hat das
        //oberste Verhalten auch die hchste Prioritt.
        //In diesem Fall hat "escape" also die hchste Prioritt, gefolgt
        //von "cruise". Ist gar kein Verhalten aktiv, werden einfach nur
        //die Motoren gestoppt. Das kann in diesem Beispiel aber gar nicht
        //eintreffen (knnte man allerdings tun, z.B. wenn die Akkus fast
        //leer sind)

        if(escape[STATE] != IDLE)      // Prioritt 2  - "escape"
        {
            moveCommand(escape);
        }
        else if(cruise[STATE] != IDLE) // Prioritt 1  - "cruise"
        {
            moveCommand(cruise);
        }
        else                           // Prioritt 0  - Nichts tun
        {
            //momentan tun wir hier nix!
        }

        Thread_Lock(0);  //Threadumschaltung wieder freigeben.
        Thread_Delay(5); //50ms Pause

        // Die 50ms Pause verhindert zu schnelle Wechsel zwischen verschiedenen
        // Verhaltensweisen. Kann aber bei Bedarf angepasst werden!

    }while(1);
}


//---[ Hauptprogramm ]----------------------------------------------------------
void main(void)
{

    PRO_BOT128_INIT();
    DRIVE_ON();
    ENC_LED_ON();
    ACS_INIT(6);   //ACS: Werte von 1 bis 20 ; 1=nah / 10=fern

    BEEP(100,200);
    BEEP(200,100);
    BEEP(100,200);

    //Der Thread zur berprfung von Statusnderungen wird gestartet:
    Thread_Start(THREAD_CHECK_ACS,thread_check_sensors);

    //Die Threads der Verhaltensweisen starten:
    Thread_Start(THREAD_BEHAVIOUR_CRUISE,behaviour_cruise);
    Thread_Start(THREAD_BEHAVIOUR_ESCAPE,behaviour_escape);

    //Kontroll Thread fr die Verhalten (Arbiter) starten:
    Thread_Start(THREAD_BEHAVIOUR_CONTROLLER,behaviour_controller);

    do
    {
       Thread_Delay(10);
    }while(1);

}


